@@ -8,6 +8,7 @@
  *   Mario Strasser       <mast@gmx.net>                            *
  *   David Corcoran       <corcoran@linuxnet.com>                   *
  *   Eirik A. Herskedal   <ehersked@cs.purdue.edu>                  *
+ *   Bruce Barnett        <muscle040302@grymoire.com>
  *                                                                  *
  ********************************************************************
  *                                                                  *
@@ -23,6 +24,7 @@
 #define PAM_SM_SESSION
 #define PAM_SM_PASSMSCULong32
 
+#include <stdlib.h>
 #include <security/pam_modules.h>
 #include <syslog.h>
 #include <musclecard.h>
@@ -41,7 +43,7 @@
 #define DEBUG 1
 
 /**
- * This struct contains secure data and shoul be overwritten
+ * This struct contains secure data and should be overwritten
  * before releasing!
  */
 struct secure_data {
@@ -141,7 +143,7 @@ int readRootCert(X509 **userCert, MSCTokenConnection pConnection,
   int index, rv;
   X509 *tmpCert, *rootCert;
   STACK *emlst;
-  unsigned char *userid;  
+  char *userid;  
   char *p;
 
   rv = getCardCert(pConnection, &tmpCert);
@@ -161,7 +163,7 @@ int readRootCert(X509 **userCert, MSCTokenConnection pConnection,
     if (*p == '@')
       break;
 
-  userid = (unsigned char *) malloc(sizeof(unsigned char) * index + 1 );
+  userid = malloc(sizeof(unsigned char) * index + 1 );
   strncpy(userid, sk_value(emlst, 0), index);
   X509_email_free(emlst);
 
@@ -213,7 +215,7 @@ int readRootCert(X509 **userCert, MSCTokenConnection pConnection,
   rv = checkCert(rootCert);
 
   if (rv == -1) {
-    syslog(LOG_ERR, "root certificate expiret or recoked");
+    syslog(LOG_ERR, "root certificate expired or revoked");
     pcsc_release(&pConnection);
     pam_release_data(sd);
     return PAM_AUTHINFO_UNAVAIL;
@@ -250,36 +252,62 @@ int readRootCert(X509 **userCert, MSCTokenConnection pConnection,
   return 0;
 }
 
-int readUserCert(X509 **userCert, MSCTokenConnection pConnection, 
+int readUserPubKey(EVP_PKEY **userCert, MSCTokenConnection pConnection, 
 		 struct secure_data *sd) {
-  int rv;
-  X509 *tmpCert;
-  unsigned char homeFile[200];
+  int rv = 0;
+  EVP_PKEY *tmpCert;
+  X509 *tmpCert1;
+  char homeFile[200];
+  /* tries to get a public key information from the user's public key certificate */
 
-  snprintf(homeFile, 200, "%s%s/.muscle/%s", pr.userpath, sd->user, pr.certname);  
+  /* Is userpath defined? If so, use it as a way to 'find' someone's
+   home directory (ignoring the value in the password file). 
+   I'm using this because the existence of the
+   specified file overrides the end user's default certificate in their 
+   home directory  - so userpath is defined for debugging only */
 
-  rv = getFileCert(homeFile, &tmpCert);
+  if (strlen(pr.userpath)>0) {
+	snprintf(homeFile, sizeof(homeFile), "%s%s/.muscle/%s", pr.userpath, sd->user, pr.certname);  
+  } else {
+	/* It's not defined. We have to determine the user's home directory. */
+	struct passwd *pw;
+	if ((pw=getpwnam(sd->user)) == NULL) {
+	  syslog(LOG_ERR, "su attempt to non-existing user: %s", sd->user);
+	  return -1;
+	} else
+	  snprintf(homeFile, sizeof(homeFile), "%s/.muscle/%s", pw->pw_dir, pr.certname);  
+  }
+  if (util_CheckFile(homeFile,(char *)(sd->user))) {
+	syslog(LOG_ERR, "Unsafe permissions on user certificate, file: %s: user: %s", homeFile, sd->user);
+	return -1;
+  }
+  rv = getFileCert(homeFile, &tmpCert1);
 
   if (rv == -1) {
-    /* Try to read a PEM cert */
-    rv = getFileCertPEM(homeFile, &tmpCert);
-  }
+    /* Didn't work - now try public key cert */
+    /* for UserAuth - this could also be a public key Certificate, not an X509 (signed) */
 
+	/* This makes the assumption the user created the certificate
+	   without an outside authority to sign it. */
+
+    rv = getPubKeyFromFile(homeFile, &tmpCert);
+	if (pr.debug)
+		syslog(LOG_INFO, "user certificate successfully read from %s", homeFile);
+  } else {
+	rv = checkCert(tmpCert1);
+
+	if (rv == -1) {
+	  syslog(LOG_ERR, "user certificate expired or revoked");
+	  return -1;
+	}
+	rv = getPublicKey(tmpCert1, &tmpCert);
+  }	
+  
   if (rv == -1) {
     syslog(LOG_ERR, "cannot read certificate from %s", homeFile);
-    pcsc_release(&pConnection);
-    pam_release_data(sd);
-    return PAM_AUTHINFO_UNAVAIL;
+	return -1;
   }
 
-  rv = checkCert(tmpCert);
-
-  if (rv == -1) {
-    syslog(LOG_ERR, "user certificate expired or revoked");
-    pcsc_release(&pConnection);
-    pam_release_data(sd);
-    return PAM_AUTHINFO_UNAVAIL;
-  }
 
   *userCert = tmpCert;
 
@@ -298,9 +326,9 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
   struct secure_data *sd;
   int i, rv, reader, result;
   int try_first_pass, use_first_pass;
-  unsigned char error[150];
+  char error[150];
   //unsigned char nameBuf[100];
-  X509 *userCert;
+  X509 *userCert = 0;
   EVP_PKEY *pubkey;
 
   reader = 0;
@@ -311,10 +339,10 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
   util_ReadPreferences();
 
   /* open log */
-  //openlog(LOGNAME, LOG_CONS | LOG_PID, LOG_AUTHPRIV);
+  openlog(LOGNAME, LOG_CONS | LOG_PID, LOG_AUTHPRIV);
 
   /* init data */
-  sd = (struct secure_data*)malloc(sizeof(struct secure_data));
+  sd = malloc(sizeof(struct secure_data));
   if (sd == NULL) {
     syslog(LOG_CRIT, "not enough free memory");
     return PAM_AUTHINFO_UNAVAIL;
@@ -323,12 +351,21 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
   /* init pcsc */
   rv = pcsc_init(&pConnection, reader);
   if (rv != MSC_SUCCESS) {
-    syslog(LOG_ERR, "musclecard error: %s", msc_error(rv));
+    syslog(LOG_ERR, "musclecard error during pcsc_init: %s", msc_error(rv));
     pam_release_data(sd);
+    /* if we return PAM_AUTHINFO_UNAVAIL, then we cannot use pam "auth
+       sufficient" That is, if the token is unavailable, will you
+       allow a password to be used. If we return PAM_AUTHINFO_UNAVAIL,
+       we cannot fall back on a password. Therefore, return PAM_ERR,
+       allowing password to be sufficient.
+
     return PAM_AUTHINFO_UNAVAIL;
+    */
+    return PAM_AUTH_ERR; 
+
   }
 
-  if (pr.debug) printf("Welcome\n");
+  if (pr.debug) printf("Welcome to pam_musclecard.so verification Module\n");
 
   /* get user name */
   rv = pam_get_user(pamh, &sd->user, NULL);
@@ -339,6 +376,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
     return PAM_USER_UNKNOWN;
   }
 
+  if (pr.debug) printf("User = %s\n", sd->user);
   /* get password */
   if (use_first_pass) {
     rv = pam_get_pin(pamh, sd->pin, NULL, PAM_AUTHTOK, 0);
@@ -356,8 +394,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
     return PAM_AUTHINFO_UNAVAIL;
   }
 
-  if (pr.debug)
-    printf("pin  = %s\n\n", sd->pin);
+  /*  if (pr.debug) printf("pin  = %s\n\n", sd->pin); */
 
   /* get random value */
   rv = getRandom(sd->rand, RAND_SIZE);
@@ -377,16 +414,17 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
     printf("\n\n");
   }
 
-  /* We must get the ACL for key #3 to determine which pin must
+  /* We must get the ACL for key #3 (or whatever) to determine which pin must
      be verified                                                */
 
   /* verify chv1 */
 
-  rv = MSCVerifyPIN(&pConnection, pr.pinnumber, sd->pin, PIN_SIZE);
+  rv = MSCVerifyPIN(&pConnection, pr.pinnumber, (unsigned char *)sd->pin,
+	PIN_SIZE);
   
   if (rv != MSC_SUCCESS) {
-    printf("Invalid PIN Entered\n");
-    syslog(LOG_ERR, "musclecard error: %s", msc_error(rv));
+    if (pr.debug) printf("Invalid PIN Entered\n");
+    syslog(LOG_ERR, "musclecard error during Verify PIN: %s", msc_error(rv));
     pcsc_release(&pConnection);
     pam_release_data(sd);
     return PAM_AUTHINFO_UNAVAIL;
@@ -403,8 +441,13 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
   rv = MSCComputeCrypt(&pConnection, &cryptInit, (MSCPUChar8)sd->rand,
 		       RAND_SIZE, (MSCPUChar8)sd->cipher, &cryptSize);
 
-  if (rv != MSC_SUCCESS) {
-    syslog(LOG_ERR, "musclecard error: %s", msc_error(rv));
+  if (rv == MSC_INCORRECT_P1) {
+    syslog(LOG_ERR, "musclecard error during Compute Crypt,  incorrect P1 value, certificate #%d: %s", pr.certnumber, msc_error(rv));
+    pcsc_release(&pConnection);
+    pam_release_data(sd);
+    return PAM_AUTHINFO_UNAVAIL;
+  }	else if (rv != MSC_SUCCESS) {
+    syslog(LOG_ERR, "musclecard error during Compute Crypt, certificate #%d, %s", pr.certnumber, msc_error(rv));
     pcsc_release(&pConnection);
     pam_release_data(sd);
     return PAM_AUTHINFO_UNAVAIL;
@@ -421,15 +464,17 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
 
   /* get the certs for the user and root CA */
 
-  if (pr.authmode == ROOTCERT)
+  if (pr.authmode == ROOTCERT) {
     rv = readRootCert(&userCert, pConnection, sd);
-  else
-    rv = readUserCert(&userCert, pConnection, sd);
-
-  if (rv != 0)
-    return PAM_AUTHINFO_UNAVAIL;
-
-  rv = getPublicKey(userCert, &pubkey);
+	if (rv != 0) {
+	  pcsc_release(&pConnection);
+	  pam_release_data(sd);
+        return PAM_AUTHINFO_UNAVAIL;
+	}
+    rv = getPublicKey(userCert, &pubkey);
+  } else {
+    rv = readUserPubKey(&pubkey, pConnection, sd);
+  }
 
   if (rv == -1) {
     syslog(LOG_ERR, "cannot read public key file from user certificate");
@@ -438,8 +483,23 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
     return PAM_AUTHINFO_UNAVAIL;
   }
 
-  rv = RSA_public_decrypt(128, sd->cipher, sd->plain, pubkey->pkey.rsa, 
-			   RSA_NO_PADDING);  
+  /* This section can be used to verify the user's certificate */
+#ifdef DEBUG_SPECIAL1
+  if (pr.debug) {
+	syslog(LOG_INFO, "Public key file read from user certificate");
+	printf("public exponent : %s\n", BN_bn2hex(pubkey->pkey.rsa->e));
+	printf("public modulus : %s\n", BN_bn2hex(pubkey->pkey.rsa->n));
+  }
+
+  BIO *eout=NULL;
+  eout = BIO_new(BIO_s_file());
+  BIO_set_fp(eout,stdout,BIO_NOCLOSE);
+  RSA_print(eout, pubkey->pkey.rsa, 0);
+
+#endif
+
+  rv = RSA_public_decrypt(128, (unsigned char *)sd->cipher,
+	(unsigned char *)sd->plain, pubkey->pkey.rsa, RSA_NO_PADDING);  
 
   syslog(LOG_ERR, "CLEAR %02X %02X %02X\n", (unsigned char)sd->plain[0], 
 	 (unsigned char)sd->plain[1], (unsigned char)sd->plain[0x79]);
@@ -447,7 +507,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
   if (rv == -1) {
     syslog(LOG_ERR, "cannot decode random number");
     ERR_error_string(ERR_get_error(), error);
-    printf("Error from openssl:\t %s\n", error);
+    if (pr.debug) printf("Error from openssl:\t %s\n", error);
     pcsc_release(&pConnection);
     pam_release_data(sd);
     return PAM_AUTHINFO_UNAVAIL;
@@ -462,12 +522,84 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
     printf("\n\n");
   }
 
-  /* compare original value and result from smartcard */
-  if (memcmp(sd->rand, sd->plain, RAND_SIZE) == 0) result = PAM_SUCCESS;
-  else result = PAM_AUTH_ERR;
 
+  /* Here is another debug section. It asks the card to decrypy what it sent out.
+   * This should be the same as the input.
+   * Normally this isn't used - Bruce */
+
+#ifdef DEBUG_SPECIAL
+  if (pr.debug) {
+	/* double check to make sure this works. We should be able to ask the card to decrypt this as well */
+
+/* assume the next certificate is the matching public key - This may be a bas assumption. -Bruce */
+	cryptInit.keyNum          = pr.certnumber + 1;  
+	cryptInit.cipherMode      = MSC_MODE_RSA_NOPAD;
+	cryptInit.cipherDirection = MSC_DIR_DECRYPT;
+	cryptInit.optParams       = NULL;
+	cryptInit.optParamsSize   = 0;
+	cryptSize = RAND_SIZE;
+
+	struct secure_data *sd2;
+	sd2 = malloc(sizeof(struct secure_data));
+	if (sd2 == NULL) {
+	  syslog(LOG_CRIT, "not enough free memory to initialize sd2");
+	  pam_release_data(sd);
+	  return PAM_AUTHINFO_UNAVAIL;
+	}
+	memcpy(sd2->rand,sd->cipher,RAND_SIZE);
+	rv = MSCComputeCrypt(&pConnection, &cryptInit, (MSCPUChar8)sd2->rand,
+						 RAND_SIZE, (MSCPUChar8)sd2->cipher, &cryptSize);
+
+	if (rv != MSC_SUCCESS) {
+	  
+	  syslog(LOG_ERR, "musclecard error during reverse Compute Crypt, certificate #%d: %s", cryptInit.keyNum, msc_error(rv));
+	}
+	printf("Undecoded value using key #%d  = ", cryptInit.keyNum);
+	  for (i=0; i < RAND_SIZE; i++) {
+		rv = (unsigned char)sd2->cipher[i];
+		printf("%02X", rv);
+	  }	
+	  printf("\n\n");
+  }
+	printf("Compare two values size by side: \n");
+	  for (i=0; i < 128; i+=8) {
+		printf("\t%02X%02X%02X%02X %02X%02X%02X%02X == %02X%02X%02X%02X %02X%02X%02X%02X?\n",
+			   (unsigned char)sd->rand[i],
+			   (unsigned char)sd->rand[i+1],
+			   (unsigned char)sd->rand[i+2],
+			   (unsigned char)sd->rand[i+3],
+			   (unsigned char)sd->rand[i+4],
+			   (unsigned char)sd->rand[i+5],
+			   (unsigned char)sd->rand[i+6],
+			   (unsigned char)sd->rand[i+7],
+			   (unsigned char)sd->plain[i],
+			   (unsigned char)sd->plain[i+1],
+			   (unsigned char)sd->plain[i+2],
+			   (unsigned char)sd->plain[i+3],
+			   (unsigned char)sd->plain[i+4],
+			   (unsigned char)sd->plain[i+5],
+			   (unsigned char)sd->plain[i+6],
+			   (unsigned char)sd->plain[i+7]
+			   );
+	  }	
+
+#endif
+
+  /* compare original value and result from smartcard */
+  if (memcmp(sd->rand, sd->plain, RAND_SIZE) == 0) {
+	if (pr.debug)
+	  printf("Challenge was Successfully met\n");
+	result = PAM_SUCCESS;
+  } else {
+	if (pr.debug)
+	  syslog(LOG_ERR, "musclecard challenge failed for user %s", sd->user);
+
+	result = PAM_AUTH_ERR; 
+  }  
+  
   /* Release certificates */
-  X509_free(userCert);
+  if (userCert)
+	X509_free(userCert);
 
   /* release pcsc */
   pcsc_release(&pConnection);
@@ -552,7 +684,7 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc,
   }
 
   /* init data */
-  sd = (struct secure_data*)malloc(sizeof(struct secure_data));
+  sd = malloc(sizeof(struct secure_data));
   if (sd == NULL) {
     syslog(LOG_CRIT, "not enough free memory");
     return PAM_AUTHINFO_UNAVAIL;
@@ -625,7 +757,8 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc,
   }
 
   /* try to change pin */
-  rv = MSCChangePIN(&pConnection, 1, sd->pin, PIN_SIZE, sd->new1, PIN_SIZE);
+  rv = MSCChangePIN(&pConnection, 1, (unsigned char *)sd->pin, PIN_SIZE,
+	(unsigned char *)sd->new1, PIN_SIZE);
 
   if (rv != MSC_SUCCESS) {
     syslog(LOG_ERR, "pc/sc error: %s", pcsc_stringify_error(rv));
