@@ -1,6 +1,6 @@
 /******************************************************************************
 ** 
-**  $Id: cryptoki.h,v 1.1.1.1 2003/10/03 19:11:56 rousseau Exp $
+**  $Id: cryptoki.h,v 1.7 2006-11-25 20:55:04 rousseau Exp $
 **
 **  Package: PKCS-11
 **  Author : Chris Osgood <oznet@mac.com>
@@ -17,6 +17,7 @@
 ** Include all "standard" RSA PKCS #11 headers
 ******************************************************************************/
 #ifndef WIN32
+#include "misc.h"
 #include "cryptoki_unix.h"
 #else
 #include "cryptoki_win32.h"
@@ -60,6 +61,7 @@
 /* #define P11_LOG(x,...) */
 #endif
 
+#define LOG_NONE   10
 #define LOG_LOW     0
 #define LOG_MED     5
 #define LOG_HIGH    9
@@ -200,10 +202,13 @@ typedef struct
     P11_Object *objects;            /* List of objects                   */
     P11_MechInfo *mechanisms;       /* List of mechanisms                */
     P11_Pin pins[2];                /* Array of cached PIN's             */
+    CK_FLAGS flags;                 /* Some information bits (see below) */
     MSCStatusInfo status_info;      /* Status of token                   */
     MSCTokenConnection conn;        /* Connection to token               */
 } P11_Slot;
 
+#define P11_SLOT_REREAD_OBJECTS        1
+
 /* A session with one slot.  */
 typedef struct _P11_Session
 {
@@ -474,11 +479,7 @@ CK_ULONG util_strpadlen(CK_CHAR *string, CK_ULONG max_len);
    CK_RV util_PadStrSet(CK_CHAR *string, CK_CHAR *value, CK_ULONG size);
    CK_RV util_StripPKCS1(CK_BYTE *data, CK_ULONG len, CK_BYTE *output, CK_ULONG *out_len);
 #ifndef __USE_GNU
-#ifndef WIN32
-   size_t strnlen(__const char *__string, size_t __maxlen);
-#else
    size_t strnlen(const char *__string, size_t __maxlen);
-#endif
 #endif /* __USE_GNU */
 CK_BBOOL util_IsLittleEndian();
 
