@@ -1,6 +1,6 @@
 /******************************************************************************
 ** 
-**  $Id: p11_object.c,v 1.1.1.2 2004/05/31 10:43:11 rousseau Exp $
+**  $Id: p11_object.c,v 1.7 2006-11-25 20:55:04 rousseau Exp $
 **
 **  Package: PKCS-11
 **  Author : Chris Osgood <oznet@mac.com>
@@ -505,17 +505,40 @@ CK_DEFINE_FUNCTION(CK_RV, C_FindObjectsInit)
             /* Fixme: Big problem? May need to malloc the pValue for the local search attributes. */
             /*        Otherwise it uses the memory pointer back into the calling application. */
             session->search_attrib = (CK_ATTRIBUTE *)calloc(ulCount, sizeof(CK_ATTRIBUTE));
-            if (!session->search_attrib)
+            if (NULL == session->search_attrib) {
                 rv = CKR_HOST_MEMORY;
+				goto finish;
+			}
             else
             {
+		CK_ULONG i;
                 memcpy(session->search_attrib, pTemplate, ulCount * sizeof(CK_ATTRIBUTE));
+		for (i=0; i<ulCount; i++) {
+		    session->search_attrib[i].pValue = 
+			malloc(session->search_attrib[i].ulValueLen);
+		    if (NULL == session->search_attrib[i].pValue) {
+			/* avoid leaking memory here */
+			CK_ULONG k; 
+			for (k=0 ; k<i ; k++)
+			    free(session->search_attrib[k].pValue);
+
+			free(session->search_attrib);
+			session->search_attrib = NULL;
+
+			rv = CKR_HOST_MEMORY;
+			goto finish;
+		    }
+		    memcpy(session->search_attrib[i].pValue, 
+			   pTemplate[i].pValue, pTemplate[i].ulValueLen);
+		}
                 session->search_attrib_count = ulCount;
             }
         }
 
-        if (st.prefs.multi_app || !slot->objects)
+        if (st.prefs.multi_app || !slot->objects || (slot->flags & P11_SLOT_REREAD_OBJECTS))
         {
+            slot->flags &= ~P11_SLOT_REREAD_OBJECTS;
+
             if (!CKR_ERROR(rv = slot_BeginTransaction(session->session.slotID)))
             {
                 msc_rv = msc_ListKeys(&slot->conn, MSC_SEQUENCE_RESET, &keyInfo);
@@ -659,6 +682,15 @@ CK_DEFINE_FUNCTION(CK_RV, C_FindObjectsFinal)
 
         if (session->search_attrib)
         {
+	    CK_ULONG j;
+	    for (j=0 ; j<session->search_attrib_count; j++) {
+		if (session->search_attrib[j].pValue) {
+		    /* redundant if, actually */
+		    free(session->search_attrib[j].pValue);
+		    session->search_attrib[j].pValue = NULL;
+		}
+	    }
+
             free(session->search_attrib);
             session->search_attrib = 0x00;
         }
