@@ -1,6 +1,6 @@
 /******************************************************************************
 ** 
-**  $Id: p11x_object.c,v 1.1.1.2 2004/05/31 10:43:11 rousseau Exp $
+**  $Id: p11x_object.c,v 1.7 2006-11-25 20:55:04 rousseau Exp $
 **
 **  Package: PKCS-11
 **  Author : Chris Osgood <oznet@mac.com>
@@ -12,29 +12,27 @@
 
 #include "cryptoki.h"
 #include "crackcert.h"
-#include <assert.h>
 
 /* #include <openssl/x509.h> */
 
 /******************************************************************************
 ** Function: object_FreeAllObjects
 **
-** Recursively releases all objects from the given slot.
+** Releases all objects from the given slot.
 **
 ** Parameters:
 **  slotID - Slot number to release
-**  list   - Object to free (used for recursion)
+**  list   - Object to free
 **
 ** Returns:
 **  none
 *******************************************************************************/
 void object_FreeAllObjects(CK_SLOT_ID slotID, P11_Object *list)
 {
-    if (list)
-    {
-        if (list->next) 
-            object_FreeAllObjects(slotID, list->next);
-        
+	P11_Object *next = NULL;
+	for (; list; list = next)
+	{
+        next = list->next;
         object_FreeObject(slotID, list);
     }
 }
@@ -99,25 +97,25 @@ void object_FreeObject(CK_SLOT_ID slotID, P11_Object *object)
 ** Deletes/frees all attributes relating to a specific object
 **
 ** Parameters:
-**  list - attribute to free (used for recursion)
+**  list - attribute to free
 **
 ** Returns:
 **  none
 *******************************************************************************/
 void object_FreeAllAttributes(P11_Attrib *list)
 {
-    if (list)
+    P11_Attrib *next;
+    for (; list; list = next)
     {
+	next = list->next;
         log_Log(LOG_LOW, "Removing attribute: %lX", list);
-
-        if (list->next) 
-            object_FreeAllAttributes(list->next);
-
-        if (list->attrib.pValue)
+ 
+        if (list->attrib.pValue) {
+	    memset(list->attrib.pValue, 0x00, list->attrib.ulValueLen);
             free(list->attrib.pValue);
+	}
 
         memset(list, 0x00, sizeof(P11_Attrib));
-
         free(list);
     }
 }
@@ -703,6 +701,7 @@ CK_RV object_MatchAttrib(CK_ATTRIBUTE *attrib, P11_Object *object)
         log_Log(LOG_LOW, "Match attribute type: 0x%lX", attrib->type);
 
         if (!CKR_ERROR(rv = object_GetAttrib(attrib->type, object, &obj_attrib)) &&
+            (obj_attrib->attrib.pValue != NULL) &&
             (!memcmp(forward, obj_attrib->attrib.pValue, attrib->ulValueLen) ||
              !memcmp(reverse, obj_attrib->attrib.pValue, attrib->ulValueLen)))
         {
@@ -712,7 +711,7 @@ CK_RV object_MatchAttrib(CK_ATTRIBUTE *attrib, P11_Object *object)
         {
             {
               CK_BYTE *buf; 
-              int len = attrib->ulValueLen;
+              CK_ULONG len = attrib->ulValueLen;
 
               /* be sure object_GetAttrib() worked before dereferencing
                * obj_attrib */
@@ -1011,6 +1010,11 @@ void object_LogAttribute(CK_ATTRIBUTE *attrib)
             log_Log(LOG_LOW, "CKA_LABEL:%s", buf);
             log_Log(LOG_LOW, "CKA_LABEL (string): %.*s", attrib->ulValueLen, attrib->pValue);
             break;
+		case CKA_APPLICATION:
+			log_Log(LOG_LOW, "CKA_APPLICATION:%s", buf);
+			log_Log(LOG_LOW, "CKA_APPLICATION (string): %.*s",
+				attrib->ulValueLen, attrib->pValue);
+			break;
         case CKA_VALUE:
             log_Log(LOG_LOW, "CKA_VALUE:%s", buf);
             break;
@@ -1053,6 +1057,18 @@ void object_LogAttribute(CK_ATTRIBUTE *attrib)
         case CKA_SENSITIVE:
             log_Log(LOG_LOW, "CKA_SENSITIVE:%s", buf);
             break;
+	 case CKA_ENCRYPT:
+            log_Log(LOG_LOW, "CKA_ENCRYPT:%s", buf);
+            break;
+	 case CKA_WRAP:
+            log_Log(LOG_LOW, "CKA_WRAP:%s", buf);
+            break;
+	 case CKA_VERIFY:
+            log_Log(LOG_LOW, "CKA_VERIFY:%s", buf);
+            break;
+	 case CKA_MODULUS_BITS:
+            log_Log(LOG_LOW, "CKA_MODULUS_BITS:%s", buf);
+            break;
         default:
             log_Log(LOG_LOW, "CKA_UNKNOWN (0x%lX):%s", attrib->type, buf);
             break;
@@ -1453,12 +1469,11 @@ CK_RV object_WriteAttributes(P11_Session *session, P11_Object *object)
         ck_attrib.pValue = &priv;
         ck_attrib.ulValueLen = sizeof(priv);
 
-/*
+
         if (object_MatchAttrib(&ck_attrib, object))
             objACL.readPermission = (MSCUShort16)object_MapPIN(st.prefs.user_pin_num);
         else
-*/
-        objACL.readPermission = MSC_AUT_ALL;
+            objACL.readPermission = MSC_AUT_ALL;
 
         objACL.writePermission = (MSCUShort16)object_MapPIN(st.prefs.user_pin_num);
         objACL.deletePermission = (MSCUShort16)object_MapPIN(st.prefs.user_pin_num);
@@ -1753,7 +1768,6 @@ CertField field)
             item = &issuer;
             break;
           default:
-            assert(0);
             return CKR_FUNCTION_FAILED;
         }
         memcpy(out, item->data, item->len);
@@ -1944,6 +1958,7 @@ CK_RV object_CreateCertificate(P11_Session *session, P11_Object *object)
     P11_Object *object_l;
     P11_Attrib *attrib;
     CK_BYTE tag, last;
+    CK_BYTE classRevCheck[sizeof(CK_ULONG)];    
 
     if (!(object->msc_obj = (MSCObjectInfo *)calloc(1, sizeof(MSCObjectInfo))))
         rv = CKR_HOST_MEMORY;
@@ -1959,10 +1974,16 @@ CK_RV object_CreateCertificate(P11_Session *session, P11_Object *object)
                 object_l->msc_obj &&
                 !CKR_ERROR(object_GetAttrib(CKA_CLASS, object_l, &attrib)))
             {
-                if (*((CK_ULONG *)attrib->attrib.pValue) == CKO_CERTIFICATE)
+                memcpy(classRevCheck, attrib->attrib.pValue, sizeof(CK_ULONG));
+                util_byterev(classRevCheck, sizeof(CK_ULONG));
+
+                if ((*((CK_ULONG *)attrib->attrib.pValue) == CKO_CERTIFICATE) ||
+                   (*((CK_ULONG *)classRevCheck) == CKO_CERTIFICATE))
                 {
                     tag = object_l->msc_obj->objectID[0];
-                    last = object_l->msc_obj->objectID[1];
+                    // Really need to find highest...
+                    if (last < object_l->msc_obj->objectID[1]) 
+                        last = object_l->msc_obj->objectID[1];
                 }
             }
 
@@ -2231,6 +2252,7 @@ CK_RV object_CreateObject(P11_Session *session, P11_Object *object)
     P11_Object *object_l;
     P11_Attrib *attrib;
     CK_BYTE tag, last;
+    CK_BYTE classRevCheck[sizeof(CK_ULONG)];      
 
     if (!(object->msc_obj = (MSCObjectInfo *)calloc(1, sizeof(MSCObjectInfo))))
         rv = CKR_HOST_MEMORY;
@@ -2246,11 +2268,18 @@ CK_RV object_CreateObject(P11_Session *session, P11_Object *object)
                 object_l->msc_obj &&
                 !CKR_ERROR(object_GetAttrib(CKA_CLASS, object_l, &attrib)))
             {
-                if (*((CK_ULONG *)attrib->attrib.pValue) == CKO_CERTIFICATE)
-                {
+                memcpy(classRevCheck, attrib->attrib.pValue, sizeof(CK_ULONG));
+                util_byterev(classRevCheck, sizeof(CK_ULONG));
+
+                if ((*((CK_ULONG *)attrib->attrib.pValue) == CKO_DATA) ||
+                   (*((CK_ULONG *)classRevCheck) == CKO_DATA))
+                   {
+            
                     tag = object_l->msc_obj->objectID[0];
-                    last = object_l->msc_obj->objectID[1];
-                }
+                    // Really need to find highest...
+                    if (last < object_l->msc_obj->objectID[1]) 
+                        last = object_l->msc_obj->objectID[1];
+                   }
             }
 
             object_l = object_l->next;
