@@ -1,6 +1,6 @@
 /******************************************************************************
 ** 
-**  $Id: p11x_slot.c,v 1.1.1.2 2004/05/31 10:43:11 rousseau Exp $
+**  $Id: p11x_slot.c,v 1.7 2006-11-25 20:55:04 rousseau Exp $
 **
 **  Package: PKCS-11
 **  Author : Chris Osgood <oznet@mac.com>
@@ -469,7 +469,8 @@ CK_RV slot_UpdateMechanisms(CK_ULONG slotID)
                                        CKF_VERIFY_RECOVER;
                     
                     if (temp_cap & MSC_CAPABLE_RSA_NOPAD)
-                         mech->info.flags =  CKF_WRAP |                                                                                CKF_UNWRAP;
+                         mech->info.flags |=  CKF_WRAP |
+                                              CKF_UNWRAP;
                 }
             }
         }
@@ -882,9 +883,12 @@ CK_RV slot_DisconnectSlot(CK_ULONG slotID, CK_ULONG action)
         while (session_l)
         {
             if (session_l->session.slotID == slotID)
+			{
                 session_FreeSession(session_l);
-
-            session_l = st.sessions;
+				session_l = st.sessions;
+			}
+			else
+				session_l = session_l->next;
         }
 
         object_FreeAllObjects(slotID, st.slots[slotID - 1].objects);
@@ -931,6 +935,8 @@ CK_RV slot_PublicMode(CK_ULONG slotID)
     else
     {
         st.slots[slotID - 1].pin_state = 0; /* Fixme: create #define for this */
+		/* mark the slot to update its objects and keys */
+		st.slots[slotID - 1].flags |= P11_SLOT_REREAD_OBJECTS;
 
         session_l = st.sessions;
         while (session_l)
@@ -984,6 +990,8 @@ CK_RV slot_UserMode(CK_ULONG slotID)
     else
     {
         st.slots[slotID - 1].pin_state = 1; /* Fixme: create #define for this */
+		/* mark the slot to update its objects and keys */
+		st.slots[slotID - 1].flags |= P11_SLOT_REREAD_OBJECTS;
 
         session_l = st.sessions;
         while (session_l)
